@@ -17,7 +17,6 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with python-mpd2.  If not, see <http://www.gnu.org/licenses/>.
 
-from collections import Callable
 import logging
 import socket
 import sys
@@ -28,7 +27,7 @@ import warnings
 # constants
 ###############################################################################
 
-VERSION = (1, 0, 0)
+VERSION = (1, 1, 0)
 HELLO_PREFIX = "OK MPD "
 ERROR_PREFIX = "ACK "
 SUCCESS = "OK"
@@ -121,8 +120,10 @@ class mpd_commands(object):
         self.commands = commands
         self.is_direct = kwargs.pop('is_direct', False)
         if kwargs:
-            raise AttributeError("mpd_commands() got unexpected keyword"
-                    " arguments %s" % ",".join(kwargs))
+            raise AttributeError(
+                "mpd_commands() got unexpected keyword"
+                " arguments %s" % ",".join(kwargs)
+            )
 
     def __call__(self, ob):
         ob.mpd_commands = self.commands
@@ -219,10 +220,12 @@ class MPDClientBase(object):
         for line in lines:
             yield self._parse_pair(line, separator)
 
-    def _parse_objects(self, lines, delimiters=[]):
+    def _parse_objects(self, lines, delimiters=[], lookup_delimiter=False):
         obj = {}
         for key, value in self._parse_pairs(lines):
             key = key.lower()
+            if lookup_delimiter and not delimiters:
+                delimiters = [key]
             if obj:
                 if key in delimiters:
                     yield obj
@@ -255,8 +258,7 @@ class MPDClientBase(object):
     def _parse_changes(self, lines):
         return self._parse_objects_direct(lines, ["cpos"])
 
-    @mpd_commands('listall', 'listallinfo', 'listfiles', 'lsinfo',
-            is_direct=True)
+    @mpd_commands('listall', 'listallinfo', 'listfiles', 'lsinfo', is_direct=True)
     def _parse_database(self, lines):
         return self._parse_objects_direct(lines, ["file", "directory", "playlist"])
 
@@ -272,7 +274,7 @@ class MPDClientBase(object):
         return pairs[0][1]
 
     @mpd_commands(
-        'channels', 'commands', 'list', 'listplaylist', 'notcommands',
+        'channels', 'commands', 'listplaylist', 'notcommands',
         'tagtypes', 'urlhandlers')
     def _parse_list(self, lines):
         seen = None
@@ -284,6 +286,11 @@ class MPDClientBase(object):
                 seen = key
             yield value
 
+    @mpd_commands('list', is_direct=True)
+    def _parse_list_groups(self, lines):
+        lines = iter(lines)
+        return self._parse_objects_direct(lines, lookup_delimiter=True)
+
     @mpd_commands('readmessages', is_direct=True)
     def _parse_messages(self, lines):
         return self._parse_objects_direct(lines, ["channel"])
@@ -360,8 +367,9 @@ class MPDClientBase(object):
 def _create_callback(self, function, wrap_result):
     """Create MPD command related response callback.
     """
-    if not isinstance(function, Callable):
+    if not callable(function):
         return None
+
     def command_callback():
         # command result callback expects response from MPD as iterable lines,
         # thus read available lines from socket
@@ -396,6 +404,7 @@ class MPDClient(MPDClientBase):
     _timeout = None
     _wrap_iterator_parsers = [
         MPDClientBase._parse_list,
+        MPDClientBase._parse_list_groups,
         MPDClientBase._parse_playlist,
         MPDClientBase._parse_changes,
         MPDClientBase._parse_songs,
@@ -450,7 +459,7 @@ class MPDClient(MPDClientBase):
             raise PendingCommandError(
                 "'{}' is not the currently pending command".format(command))
         del self._pending[0]
-        if isinstance(retval, Callable):
+        if callable(retval):
             return retval()
         return retval
 
@@ -462,14 +471,14 @@ class MPDClient(MPDClientBase):
             raise PendingCommandError(
                 "Cannot execute '{}' with pending commands".format(command))
         if self._command_list is not None:
-            if not isinstance(retval, Callable):
+            if not callable(retval):
                 raise CommandListError(
                     "'{}' not allowed in command list".format(command))
             self._write_command(command, args)
             self._command_list.append(retval)
         else:
             self._write_command(command, args)
-            if isinstance(retval, Callable):
+            if callable(retval):
                 return retval()
             return retval
 
@@ -592,6 +601,7 @@ class MPDClient(MPDClientBase):
             sock = None
             try:
                 sock = socket.socket(af, socktype, proto)
+                sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
                 sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
                 sock.settimeout(self.timeout)
                 sock.connect(sa)
@@ -658,7 +668,7 @@ class MPDClient(MPDClientBase):
         try:
             helloline = self._rfile.readline()
             self._hello(helloline)
-        except:
+        except Exception:
             self.disconnect()
             raise
 
